@@ -21,6 +21,7 @@
 #include <linux/posix_acl_xattr.h>
 #include <linux/uio.h>
 #include <linux/fiemap.h>
+#include <linux/pagemap.h>
 
 struct shiftfs_super_info {
 	struct vfsmount *mnt;
@@ -317,7 +318,7 @@ static int shiftfs_setxattr(struct dentry *dentry, struct inode *inode,
 	const struct cred *oldcred;
 
 	oldcred = shiftfs_override_creds(dentry->d_sb);
-	err = vfs_setxattr(lowerd, name, value, size, flags);
+	err = vfs_setxattr(&init_user_ns, lowerd, name, value, size, flags);
 	revert_creds(oldcred);
 
 	shiftfs_copyattr(lowerd->d_inode, inode);
@@ -334,7 +335,7 @@ static int shiftfs_xattr_get(const struct xattr_handler *handler,
 	const struct cred *oldcred;
 
 	oldcred = shiftfs_override_creds(dentry->d_sb);
-	err = vfs_getxattr(lowerd, name, value, size);
+	err = vfs_getxattr(&init_user_ns, lowerd, name, value, size);
 	revert_creds(oldcred);
 
 	return err;
@@ -354,14 +355,15 @@ static ssize_t shiftfs_listxattr(struct dentry *dentry, char *list,
 	return err;
 }
 
-static int shiftfs_removexattr(struct dentry *dentry, const char *name)
+static int shiftfs_removexattr(struct user_namespace *ns,
+			       struct dentry *dentry, const char *name)
 {
 	struct dentry *lowerd = dentry->d_fsdata;
 	int err;
 	const struct cred *oldcred;
 
 	oldcred = shiftfs_override_creds(dentry->d_sb);
-	err = vfs_removexattr(lowerd, name);
+	err = vfs_removexattr(&init_user_ns, lowerd, name);
 	revert_creds(oldcred);
 
 	/* update c/mtime */
@@ -371,12 +373,13 @@ static int shiftfs_removexattr(struct dentry *dentry, const char *name)
 }
 
 static int shiftfs_xattr_set(const struct xattr_handler *handler,
+			     struct user_namespace *ns,
 			     struct dentry *dentry, struct inode *inode,
 			     const char *name, const void *value, size_t size,
 			     int flags)
 {
 	if (!value)
-		return shiftfs_removexattr(dentry, name);
+		return shiftfs_removexattr(&init_user_ns, dentry, name);
 	return shiftfs_setxattr(dentry, inode, name, value, size, flags);
 }
 
@@ -453,7 +456,7 @@ static int shiftfs_create_object(struct inode *diri, struct dentry *dentry,
 		inode->i_state |= I_CREATING;
 		spin_unlock(&inode->i_lock);
 
-		inode_init_owner(inode, diri, mode);
+		inode_init_owner(&init_user_ns, inode, diri, mode);
 		modei = inode->i_mode;
 	}
 
@@ -464,22 +467,22 @@ static int shiftfs_create_object(struct inode *diri, struct dentry *dentry,
 
 	if (hardlink) {
 		lowerd_link = hardlink->d_fsdata;
-		err = vfs_link(lowerd_link, loweri_dir, lowerd_new, NULL);
+		err = vfs_link(lowerd_link, &init_user_ns, loweri_dir, lowerd_new, NULL);
 	} else {
 		switch (modei & S_IFMT) {
 		case S_IFDIR:
-			err = vfs_mkdir(loweri_dir, lowerd_new, modei);
+			err = vfs_mkdir(&init_user_ns, loweri_dir, lowerd_new, modei);
 			break;
 		case S_IFREG:
-			err = vfs_create(loweri_dir, lowerd_new, modei, excl);
+			err = vfs_create(&init_user_ns, loweri_dir, lowerd_new, modei, excl);
 			break;
 		case S_IFLNK:
-			err = vfs_symlink(loweri_dir, lowerd_new, symlink);
+			err = vfs_symlink(&init_user_ns, loweri_dir, lowerd_new, symlink);
 			break;
 		case S_IFSOCK:
 			/* fall through */
 		case S_IFIFO:
-			err = vfs_mknod(loweri_dir, lowerd_new, modei, 0);
+			err = vfs_mknod(&init_user_ns, loweri_dir, lowerd_new, modei, 0);
 			break;
 		default:
 			err = -EINVAL;
@@ -535,7 +538,8 @@ out_iput:
 	return err;
 }
 
-static int shiftfs_create(struct inode *dir, struct dentry *dentry,
+static int shiftfs_create(struct user_namespace *ns,
+			  struct inode *dir, struct dentry *dentry,
 			  umode_t mode,  bool excl)
 {
 	mode |= S_IFREG;
@@ -543,7 +547,7 @@ static int shiftfs_create(struct inode *dir, struct dentry *dentry,
 	return shiftfs_create_object(dir, dentry, mode, NULL, NULL, excl);
 }
 
-static int shiftfs_mkdir(struct inode *dir, struct dentry *dentry,
+static int shiftfs_mkdir(struct user_namespace *ns, struct inode *dir, struct dentry *dentry,
 			 umode_t mode)
 {
 	mode |= S_IFDIR;
@@ -557,7 +561,8 @@ static int shiftfs_link(struct dentry *hardlink, struct inode *dir,
 	return shiftfs_create_object(dir, dentry, 0, NULL, hardlink, false);
 }
 
-static int shiftfs_mknod(struct inode *dir, struct dentry *dentry, umode_t mode,
+static int shiftfs_mknod(struct user_namespace *ns,
+			 struct inode *dir, struct dentry *dentry, umode_t mode,
 			 dev_t rdev)
 {
 	if (!S_ISFIFO(mode) && !S_ISSOCK(mode))
@@ -566,7 +571,7 @@ static int shiftfs_mknod(struct inode *dir, struct dentry *dentry, umode_t mode,
 	return shiftfs_create_object(dir, dentry, mode, NULL, NULL, false);
 }
 
-static int shiftfs_symlink(struct inode *dir, struct dentry *dentry,
+static int shiftfs_symlink(struct user_namespace *ns, struct inode *dir, struct dentry *dentry,
 			   const char *symlink)
 {
 	return shiftfs_create_object(dir, dentry, S_IFLNK, symlink, NULL, false);
@@ -584,9 +589,9 @@ static int shiftfs_rm(struct inode *dir, struct dentry *dentry, bool rmdir)
 	oldcred = shiftfs_override_creds(dentry->d_sb);
 	inode_lock_nested(loweri, I_MUTEX_PARENT);
 	if (rmdir)
-		err = vfs_rmdir(loweri, lowerd);
+		err = vfs_rmdir(&init_user_ns, loweri, lowerd);
 	else
-		err = vfs_unlink(loweri, lowerd, NULL);
+		err = vfs_unlink(&init_user_ns, loweri, lowerd, NULL);
 	revert_creds(oldcred);
 
 	if (!err) {
@@ -615,7 +620,8 @@ static int shiftfs_rmdir(struct inode *dir, struct dentry *dentry)
 	return shiftfs_rm(dir, dentry, true);
 }
 
-static int shiftfs_rename(struct inode *olddir, struct dentry *old,
+static int shiftfs_rename(struct user_namespace *ns,
+			  struct inode *olddir, struct dentry *old,
 			  struct inode *newdir, struct dentry *new,
 			  unsigned int flags)
 {
@@ -625,6 +631,14 @@ static int shiftfs_rename(struct inode *olddir, struct dentry *old,
 		      *trapd;
 	struct inode *loweri_dir_old = lowerd_dir_old->d_inode,
 		     *loweri_dir_new = lowerd_dir_new->d_inode;
+	struct renamedata rd = {
+		.old_mnt_userns	= ns,
+		.old_dir	= loweri_dir_old,
+		.old_dentry	= lowerd_old,
+		.new_mnt_userns	= ns,
+		.new_dir	= loweri_dir_new,
+		.new_dentry	= lowerd_new,
+	};
 	int err = -EINVAL;
 	const struct cred *oldcred;
 
@@ -634,8 +648,7 @@ static int shiftfs_rename(struct inode *olddir, struct dentry *old,
 		goto out_unlock;
 
 	oldcred = shiftfs_override_creds(old->d_sb);
-	err = vfs_rename(loweri_dir_old, lowerd_old, loweri_dir_new, lowerd_new,
-			 NULL, flags);
+	err = vfs_rename(&rd);
 	revert_creds(oldcred);
 
 	shiftfs_copyattr(loweri_dir_old, olddir);
@@ -691,7 +704,7 @@ out:
 	return d_splice_alias(inode, dentry);
 }
 
-static int shiftfs_permission(struct inode *inode, int mask)
+static int shiftfs_permission(struct user_namespace *ns, struct inode *inode, int mask)
 {
 	int err;
 	const struct cred *oldcred;
@@ -702,12 +715,12 @@ static int shiftfs_permission(struct inode *inode, int mask)
 		return -ECHILD;
 	}
 
-	err = generic_permission(inode, mask);
+	err = generic_permission(&init_user_ns, inode, mask);
 	if (err)
 		return err;
 
 	oldcred = shiftfs_override_creds(inode->i_sb);
-	err = inode_permission(loweri, mask);
+	err = inode_permission(&init_user_ns, loweri, mask);
 	revert_creds(oldcred);
 
 	return err;
@@ -733,7 +746,8 @@ static int shiftfs_fiemap(struct inode *inode,
 	return err;
 }
 
-static int shiftfs_tmpfile(struct inode *dir, struct dentry *dentry,
+static int shiftfs_tmpfile(struct user_namespace *ns,
+			   struct inode *dir, struct dentry *dentry,
 			   umode_t mode)
 {
 	int err;
@@ -745,13 +759,13 @@ static int shiftfs_tmpfile(struct inode *dir, struct dentry *dentry,
 		return -EOPNOTSUPP;
 
 	oldcred = shiftfs_override_creds(dir->i_sb);
-	err = loweri->i_op->tmpfile(loweri, lowerd, mode);
+	err = loweri->i_op->tmpfile(&init_user_ns, loweri, lowerd, mode);
 	revert_creds(oldcred);
 
 	return err;
 }
 
-static int shiftfs_setattr(struct dentry *dentry, struct iattr *attr)
+static int shiftfs_setattr(struct user_namespace *ns, struct dentry *dentry, struct iattr *attr)
 {
 	struct dentry *lowerd = dentry->d_fsdata;
 	struct inode *loweri = lowerd->d_inode;
@@ -761,7 +775,7 @@ static int shiftfs_setattr(struct dentry *dentry, struct iattr *attr)
 	struct shiftfs_super_info *sbinfo = sb->s_fs_info;
 	int err;
 
-	err = setattr_prepare(dentry, attr);
+	err = setattr_prepare(&init_user_ns, dentry, attr);
 	if (err)
 		return err;
 
@@ -778,7 +792,7 @@ static int shiftfs_setattr(struct dentry *dentry, struct iattr *attr)
 
 	inode_lock(loweri);
 	oldcred = shiftfs_override_creds(dentry->d_sb);
-	err = notify_change(lowerd, &newattr, NULL);
+	err = notify_change(&init_user_ns, lowerd, &newattr, NULL);
 	revert_creds(oldcred);
 	inode_unlock(loweri);
 
@@ -787,7 +801,8 @@ static int shiftfs_setattr(struct dentry *dentry, struct iattr *attr)
 	return err;
 }
 
-static int shiftfs_getattr(const struct path *path, struct kstat *stat,
+static int shiftfs_getattr(struct user_namespace *ns,
+			   const struct path *path, struct kstat *stat,
 			   u32 request_mask, unsigned int query_flags)
 {
 	struct inode *inode = path->dentry->d_inode;
@@ -870,9 +885,9 @@ shift_acl_xattr_ids(struct user_namespace *from, struct user_namespace *to,
 			entry->e_id = cpu_to_le32(from_kuid(&init_user_ns, kuid));
 			break;
 		case ACL_GROUP:
-			kgid = make_kgid(&init_user_ns, le32_to_cpu(entry->e_id));
+			kgid = make_kgid(from, le32_to_cpu(entry->e_id));
 			kgid = shift_kgid(from, to, kgid);
-			entry->e_id = cpu_to_le32(from_kgid(&init_user_ns, kgid));
+			entry->e_id = cpu_to_le32(from_kgid(from, kgid));
 			break;
 		default:
 			break;
@@ -880,7 +895,8 @@ shift_acl_xattr_ids(struct user_namespace *from, struct user_namespace *to,
 	}
 }
 
-static struct posix_acl *shiftfs_get_acl(struct inode *inode, int type)
+static struct posix_acl *
+shiftfs_get_acl(struct inode *inode, int type, bool rcu)
 {
 	struct inode *loweri = inode->i_private;
 	const struct cred *oldcred;
@@ -890,6 +906,9 @@ static struct posix_acl *shiftfs_get_acl(struct inode *inode, int type)
 	int size;
 	int err;
 
+	if (rcu)
+		return ERR_PTR(-ECHILD);
+
 	if (!IS_POSIXACL(loweri))
 		return NULL;
 
@@ -941,6 +960,7 @@ shiftfs_posix_acl_xattr_get(const struct xattr_handler *handler,
 
 static int
 shiftfs_posix_acl_xattr_set(const struct xattr_handler *handler,
+			    struct user_namespace *ns,
 			    struct dentry *dentry, struct inode *inode,
 			    const char *name, const void *value,
 			    size_t size, int flags)
@@ -952,7 +972,7 @@ shiftfs_posix_acl_xattr_set(const struct xattr_handler *handler,
 		return -EOPNOTSUPP;
 	if (handler->flags == ACL_TYPE_DEFAULT && !S_ISDIR(inode->i_mode))
 		return value ? -EACCES : 0;
-	if (!inode_owner_or_capable(inode))
+	if (!inode_owner_or_capable(ns, inode))
 		return -EPERM;
 
 	if (value) {
@@ -962,7 +982,7 @@ shiftfs_posix_acl_xattr_set(const struct xattr_handler *handler,
 		err = shiftfs_setxattr(dentry, inode, handler->name, value,
 				       size, flags);
 	} else {
-		err = shiftfs_removexattr(dentry, handler->name);
+		err = shiftfs_removexattr(&init_user_ns, dentry, handler->name);
 	}
 
 	if (!err)
@@ -1420,11 +1440,11 @@ static int shiftfs_btrfs_ioctl_fd_restore(int cmd, int fd, void __user *arg,
 	else
 		ret = copy_to_user(arg, v2, sizeof(*v2));
 
-	__close_fd(current->files, fd);
+	close_fd(fd);
 	kfree(v1);
 	kfree(v2);
 
-	return ret;
+	return ret ? -EFAULT: 0;
 }
 
 static int shiftfs_btrfs_ioctl_fd_replace(int cmd, void __user *arg,
@@ -1438,6 +1458,9 @@ static int shiftfs_btrfs_ioctl_fd_replace(int cmd, void __user *arg,
 	struct btrfs_ioctl_vol_args *v1 = NULL;
 	struct btrfs_ioctl_vol_args_v2 *v2 = NULL;
 
+	*b1 = NULL;
+	*b2 = NULL;
+
 	if (!is_btrfs_snap_ioctl(cmd))
 		return 0;
 
@@ -1446,29 +1469,29 @@ static int shiftfs_btrfs_ioctl_fd_replace(int cmd, void __user *arg,
 		if (IS_ERR(v1))
 			return PTR_ERR(v1);
 		oldfd = v1->fd;
-		*b1 = v1;
 	} else {
 		v2 = memdup_user(arg, sizeof(*v2));
 		if (IS_ERR(v2))
 			return PTR_ERR(v2);
 		oldfd = v2->fd;
-		*b2 = v2;
 	}
 
 	src = fdget(oldfd);
-	if (!src.file)
-		return -EINVAL;
+	if (!src.file) {
+		ret = -EINVAL;
+		goto err_free;
+	}
 
 	ret = shiftfs_real_fdget(src.file, &lfd);
 	if (ret) {
 		fdput(src);
-		return ret;
+		goto err_free;
 	}
 
 	/*
 	 * shiftfs_real_fdget() does not take a reference to lfd.file, so
 	 * take a reference here to offset the one which will be put by
-	 * __close_fd(), and make sure that reference is put on fdput(lfd).
+	 * close_fd(), and make sure that reference is put on fdput(lfd).
 	 */
 	get_file(lfd.file);
 	lfd.flags |= FDPUT_FPUT;
@@ -1477,7 +1500,8 @@ static int shiftfs_btrfs_ioctl_fd_replace(int cmd, void __user *arg,
 	*newfd = get_unused_fd_flags(lfd.file->f_flags);
 	if (*newfd < 0) {
 		fdput(lfd);
-		return *newfd;
+		ret = *newfd;
+		goto err_free;
 	}
 
 	fd_install(*newfd, lfd.file);
@@ -1492,8 +1516,19 @@ static int shiftfs_btrfs_ioctl_fd_replace(int cmd, void __user *arg,
 		v2->fd = oldfd;
 	}
 
-	if (ret)
+	if (!ret) {
+		*b1 = v1;
+		*b2 = v2;
+	} else {
 		shiftfs_btrfs_ioctl_fd_restore(cmd, *newfd, arg, v1, v2);
+		ret = -EFAULT;
+	}
+
+	return ret;
+
+err_free:
+	kfree(v1);
+	kfree(v2);
 
 	return ret;
 }
@@ -1722,6 +1757,8 @@ const struct file_operations shiftfs_file_operations = {
 	.compat_ioctl		= shiftfs_compat_ioctl,
 	.copy_file_range	= shiftfs_copy_file_range,
 	.remap_file_range	= shiftfs_remap_file_range,
+	.splice_read		= generic_file_splice_read,
+	.splice_write		= iter_file_splice_write,
 };
 
 const struct file_operations shiftfs_dir_operations = {
